import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

import org.jdom.Attribute;
import org.jdom.Comment;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.ProcessingInstruction;
import org.jdom.input.DOMBuilder;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * <p>
 * Demonstrate building JDOM Documents from scratch and existing XML 
 *   data sources.
 * </p>
 *
 * @version 1.0
 */
public class JDOMTest {
    
    public JDOMTest() {        
    }
    
    /**
     * <p>
     * Build a JDOM <code>Document</code> from scratch
     * </p>
     *
     * @param out <code>OutputStream</code> to write created XML to
     * @throws <code>IOException</code> when output errors occur.
     */
    public void newDocument(OutputStream out) 
        throws IOException, JDOMException {

        Namespace ns = Namespace.getNamespace("linux", "http://www.linux.org");
            
        Document doc = 
          new Document(new Element("linux", "config"))             
            .setDocType(new DocType("linux:config", 
                                    "DTD/linux.dtd"))
            .addContent(
                new ProcessingInstruction("cocoon-process", 
                                          "type=\"xsp\""))
            .addContent(
                new ProcessingInstruction("cocoon-process", 
                                          "type=\"xslt\""));
           
         doc.getRootElement()
              .addAttribute("kernel", "2.2.14")         // easy way
              .addAttribute(
                  new Attribute("dist", "RedHat 6.1"))  // hard way
              .addContent(new Element("linux", "gui")
                 .setText("No Window Manager Installed"))
              .addContent(new Comment("Sound Card Configuration"))
              .addContent(new Element("sound")
                .addContent(new Comment("Sound Blaster Card"))
                .addContent(new Element("card")
                    .addContent(new Element("name")
                        .setText("Sound Blaster Platinum")))
                );
  
       XMLOutputter fmt = new XMLOutputter();
       fmt.output(doc, out);        
    }
    
    public void domDocument(File file, OutputStream out) 
        throws IOException, JDOMException {
            
        DOMBuilder builder = new DOMBuilder(true);
        Document doc = builder.build(file);
        
        XMLOutputter fmt = new XMLOutputter();
        fmt.output(doc, out);        
    }
    
    public void saxDocument(File file, OutputStream out) 
        throws IOException, JDOMException {
            
        SAXBuilder builder = new SAXBuilder(true);
        Document doc = builder.build(file);
        
        XMLOutputter fmt = new XMLOutputter();
        fmt.output(doc, out);            
    }    
    
    /**
     * <p>
     * Static entry point for JDOM testing.
     * </p>
     */
    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("Usage: JDOMTest [filename to parse]");
            System.exit(-1);
        }
        
        try {
            JDOMTest test = new JDOMTest();

            System.out.println(
                "\n\n----------------------------------------");        
            System.out.println(
                "Testing creating Document from scratch ...");
            System.out.println(
                "----------------------------------------\n\n");        
            test.newDocument(System.out);
        
            System.out.println(
                "\n\n----------------------------------------");
            System.out.println(
                "Testing reading Document using DOM ...");
            System.out.println(
                "----------------------------------------\n\n");                
            test.domDocument(new File(args[0]), System.out);
        
            System.out.println(
                "\n\n----------------------------------------");
            System.out.println(
                "Testing reading Document using SAX ...");
            System.out.println(
                "----------------------------------------\n\n");                
            test.saxDocument(new File(args[0]), System.out);        

            System.out.println(
                "\n\n----------------------------------------");        
            System.out.println(
                "Tests complete.  Successful build in place.");
            
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof JDOMException) {
                System.out.println(((JDOMException)e).getRootCause()
                                                     .getMessage());
            } else {
                System.out.println(e.getMessage());
            }
        }
    }    
    
}